// Clock must be the interrupt pin
#define CLOCK 0
#define DATA  3
#define PS_PORT portd
#define PS_TRIS trisd

void send_Data(char dataOut)
{
	char count, parity = 0;

	PS_TRIS.CLOCK = 0;		//All Outputs
	PS_TRIS.DATA = 0;

	PS_PORT.CLOCK = 0;		//Set Clock Signal Low
	delay_10us(11);			//Delay for 110u seconds

	PS_PORT.DATA = 0;		//Send start bit
	PS_TRIS.CLOCK = 1;		//Set clock to input

	for (count=0; count<8; count++)
	{
		while (test_bit (PS_PORT,CLOCK) == 1);	//While clock = 1

		PS_TRIS.DATA = dataOut & 0x01;
		parity += dataOut & 0x01;
		dataOut = dataOut >> 1;

		while (test_bit (PS_PORT,CLOCK) == 0);	//While clock = 0
	}

	while (test_bit (PS_PORT,CLOCK) == 1);	//While clock = 1

	parity = parity & 0x01;
	parity = !parity;

	PS_TRIS.DATA = parity & 0x01;

	while (test_bit (PS_PORT,CLOCK) == 0);	//While clock = 0
	while (test_bit (PS_PORT,CLOCK) == 1);	//While clock = 1

	PS_TRIS.DATA = 1;						//Set data to input

	while (test_bit (PS_PORT,DATA) == 1);	//While data = 1
	while (test_bit (PS_PORT,CLOCK) == 1);	//While clock = 1
	while (test_bit (PS_PORT,CLOCK) == 0);	//While clock = 0
	while (test_bit (PS_PORT,DATA) == 0);	//While data = 0
}


char read_Data(void)
{
	char dataIn = 0;
	char count;

	trisc = 0xFF; 							//All Inputs

	if (test_bit (PS_PORT,CLOCK) == 0 | test_bit (PS_PORT,DATA) == 0) return 0;

	for (count=0; count < 11; count++)		//Reception Loop
	{
		while (test_bit (PS_PORT,CLOCK) == 1);	//While clock = 1

		if (count == 0 && test_bit (PS_PORT,DATA) == 1) return 0; 	//Start Bit Read?
		else if (count > 0 && count < 9)
		{
			if (test_bit (PS_PORT,DATA) == 1)
			{
				dataIn = dataIn + 0x80;
			}
			dataIn = dataIn >> 1;
		}
		while (test_bit (PS_PORT,CLOCK) == 0);	//While clock = 0
	}

	return dataIn;
}

void init_Mouse(void)
{
	send_Data(0xFF);	//Reset
	read_Data();		//Read PS2 But Ignore
	send_Data(0xF5);	//Disable
	send_Data(0xF3);	//Set Sampling Rate
	send_Data(0x0A);	//Sampling Rate
	send_Data(0xE8);	//Set Resolution
	send_Data(0x03);	//Resolution Value
	send_Data(0xF0);	//Set Remote
	send_Data(0xF4);	//Enable
	send_Data(0xEB);	//Read Data
	read_Data();		//Read PS2 But Ignore
}