	title		"Rob's RGB/VGA video generator"
	subtitle	"Updated for 4MIPS (either 16MHz or 4MHz HS-PLL)"

; This file contains all code required for generation of a RGB
; video signal.  Depending on clock speed, it should be scaleable
; to approx. 120 horizontal pixels by up to 400 lines vertical -
; it all depends on memory available.

; A VGA screen consists of 450 lines.  Only 400 are visible; the
; other 50 are vertical retrace or vertical synch signal.
; You can calculate the line numbers by dividing the vertical
; time by the horizontal time.

; VERTICAL TIMING - all of these times fit within a horizontal line:
; General information - 60Hz; total of 14.27ms per frame (screen)
; Vertical synch - 0.06ms
; "Back Porch" - 1.08ms
; Video generation - 12.72ms
; "Front Porch" - 0.41ms

; Horizontal timing:
; General information - 31469Hz; total of 31.77us per line
;	3.77us = synch pulse
;	1.89us = back porch
;	25.17us = active video time
;	0.94us = front porch

; MIPS rating (varies w/ or w/o PLL mode)
MIPS	=	4
; Horizontal clock frequency
Hclock	=	31469
; Number of cycles to execute during each Hclock interval
Hinstr	=	(MIPS * 1000000) / Hclock
; Number of cycles to execute during HSYNC pulse
Hsync	=	(377 * Hinstr) / 3177
; Number of cycles to execute during each "back porch" interval
Hback	=	(189 * Hinstr) / 3177
; Number of cycles to execute during the active video interval
Hactive	=	(2517 * Hinstr) / 3177
; Number of cycles to execute during the "front porch" interval
Hfront	=	(94 * Hinstr) / 3177
; Number of bytes per screen (pairs of pixels)
; This may be a little bit of a rough estimate and may
; just need tweaking...
pixelfudge	=	21	; 8 for other instructions and 3 carryover
pixelpairs	=	(Hactive - pixelfudge) / 4
; Number of cycles used by interrupt itself plus fudge factor:
delayfudge	=	10
; Number of cycles to count for timer:
delayamt	equ		0x10000 - ((Hinstr - delayfudge) / 2)

; Macro to define memory address: (good for 64K)
daddr	macro	address
		db		high address,low address
		endm

; PORT usage:
; PORTB = 0000XRGB
; PORTC = VH000000
hpulse			=	6
vpulse			=	7

videomemory 	= 0x100		; starting address
screenheight	= d'40'
screenwidth 	= pixelpairs*2

videoTBLPTRL	=	0xf0	; video routine save point
videoTBLPTRH	=	0xf1
saveTABLAT		=	0xf2	; user save point
saveTBLPTRL		=	0xf3
saveTBLPTRH		=	0xf4
saveFSR0L	 	= 	0xf5	; per-line save point!
saveFSR0H 		=	0xf6

; This interrupt routine depends on the FAST
; return from interrupt.  No other interrupt
; may use this!
video_interrupt_handler					; Hinstr cycles max!
		; clear Timer0 flag (1 cycle)
		bcf		INTCON,TMR0IF
		; toggle HSYNCH and reset Timer0 (15 cycles)
Hsync_start
		bcf		PORTC,hpulse			;1
		movlw	high delayamt			;1  (reset timer)
		movwf	TMR0H					;1
		movlw	low	delayamt			;1
		movwf	TMR0L					;1
		movff	TBLPTRH,saveTBLPTRH		;2  (save TBL stuff)
		movff	TBLPTRL,saveTBLPTRL		;2
		movff	TABLAT,saveTABLAT		;2
		movff	videoTBLPTRL,TBLPTRL	;2  (restore TBL stuff)
		movff	videoTBLPTRH,TBLPTRH	;2  (adds 1 extra cycle!)
		bsf		PORTC,hpulse			;1
Hsync_end
Hsync_cycles = (Hsync_end - Hsync_start)/2
if (Hsync_cycles != Hsync) 
	messg	"Hsync time is incorrect.  May need to modify!"
endif
		; Read branch address from TABLAT (8 cycles)
		tblrd*+					;2 post increment
		movff	TABLAT,PCLATH	;2
		tblrd*+					;2
		movf	TABLAT,W		;1
		movwf	PCL				;1
; *** AT THIS POINT, CONTROL IS AT APPROPRIATE LINE# ***
;		0 cycles!

endofinterrupt
		; Restore status and end interrupt (2 cycles)
		movff	TBLPTRL,videoTBLPTRL	;2
		movff	TBLPTRH,videoTBLPTRH	;2
		movff	saveTBLPTRH,TBLPTRH		;2
		movff	saveTBLPTRL,TBLPTRL		;2
		movff	saveTABLAT,TABLAT		;2
		retfie FAST						;2

; This the jump datatable used for branches in the
; interrupt routine.  Each DATA entry is in HI/LO
; format.
jumptable
		daddr	vsynch_on		; 0
		daddr	vsynch_off		; 1
counter= 0
while counter<d'34'
		daddr	endofinterrupt	; 2-35
counter+= 1
endw
; This is the height of each physical pixel.
; There are 40 lines, so repeat it 40 times (400 resolution)
counter= 0
while counter<d'40'
		daddr	drawlineandsave		; 36-435
		daddr	drawlinewithreset
		daddr	drawlinewithreset
		daddr	drawlinewithreset
		daddr	drawlinewithreset
		daddr	drawlinewithreset
		daddr	drawlinewithreset
		daddr	drawlinewithreset
		daddr	drawlinewithreset
		daddr	drawlinewithreset
counter+= 1
endw
counter= 0
while counter<d'13'
		daddr	endofinterrupt	; 436-448
counter+= 1
endw
		daddr	resetTBLPTR		; 449

; Reset the TBLPTR registers.  This sets the pointer
; back to the beginning of the jump table.
resetTBLPTR
		movlw	low (jumptable)
		movwf	TBLPTRL
		movlw	high (jumptable)
		movwf	TBLPTRH
		goto	endofinterrupt

video_initialize
		; Setup PORTs:
		movlb	0
		movlw	b'000000000'	; B0-B8=output
		movwf	DDRB
		clrf	PORTB
		movwf	DDRC
		clrf	PORTC
		bsf		PORTC,vpulse
		bsf		PORTC,hpulse
		; Setup jump table position:
		movlw	low (jumptable)
		movwf	videoTBLPTRL
		movlw	high (jumptable)
		movwf	videoTBLPTRH
		; Set delay and start timer0
		movlw	high delayamt
		movwf	TMR0H
		movlw	low	delayamt
		movwf	TMR0L
		clrf	T0CON
		clrf	INTCON
		bsf		T0CON,TMR0ON
		bsf		INTCON,TMR0IE
		bsf		INTCON,GIE
		return

; Start the VSYNC pulse:
vsynch_on	; 3 cycles
		bcf		PORTC,vpulse
		goto	endofinterrupt

; End the VSYNC pulse and reset the video memory pointer in 
; preparation to the drawing that will be coming up:
vsynch_off	; 5 cycles
		bsf		PORTC,vpulse
		lfsr	FSR0,videomemory ;2
		goto	endofinterrupt

; Draw a line.  Saves FSR0 first.
; The drawing code must exactly match the other drawline routine!
drawlineandsave
		movff	FSR0L,saveFSR0L
		movff	FSR0H,saveFSR0H
counter= 0
	while counter<pixelpairs
		movf	POSTINC0,W		;1
		movwf	PORTB			;1
		swapf	WREG			;1
		movwf	PORTB			;1
counter+= 1
	endw
		nop						;1
		clrf	PORTB			;1
		goto 	endofinterrupt	;2
drawlineandsave_length = ($ - drawlineandsave) / 2

; Draw a line.  Restores FSR0 first.
; The drawing code must exactly match the other drawline routine!
drawlinewithreset
		movff	saveFSR0L,FSR0L
		movff	saveFSR0H,FSR0H
counter= 0
	while counter<pixelpairs
		movf	POSTINC0,W		;1
		movwf	PORTB			;1
		swapf	WREG			;1
		movwf	PORTB			;1
counter+= 1
	endw
		nop						;1
		clrf	PORTB			;1
		goto 	endofinterrupt	;2
drawlinewithreset_length = ($ - drawlinewithreset) / 2
